#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const fs_1 = require("fs");
const promises_1 = require("fs/promises");
const path_1 = require("path");
const yargs_1 = tslib_1.__importDefault(require("yargs"));
const helpers_1 = require("yargs/helpers");
const zod_1 = require("zod");
const Responses_1 = require("./lib/Responses");
const Encryption_1 = require("./lib/Encryption");
const _1 = require(".");
if (require.main !== module) {
    console.error(`this should be the main module`);
    process.exit(1);
}
const main = async () => {
    const print = async (outFile, data) => {
        const json = JSON.stringify(data, null, 2);
        if (outFile) {
            await fs_1.promises.writeFile(path_1.resolve(process.cwd(), outFile), json, {
                encoding: `utf-8`,
            });
        }
        else {
            console.log(data);
        }
    };
    await yargs_1.default(helpers_1.hideBin(process.argv))
        .options({
        outFile: {
            type: `string`,
            alias: `o`,
            description: `Optional output file`,
        },
    })
        .command(`authentication`, `Get authentication results`, {
        username: {
            type: `string`,
            alias: `u`,
            demandOption: true,
            description: `User name`,
        },
        password: {
            type: `string`,
            alias: `p`,
            demandOption: true,
            description: `Password`,
        },
    }, async (argv) => {
        const { username, password, outFile } = zod_1.z
            .object({
            username: zod_1.z.string(),
            password: zod_1.z.string(),
            outFile: zod_1.z.string().optional(),
        })
            .parse(argv);
        const client = await _1.Client.createClient(username, password);
        await print(outFile, await client.getAuth());
    })
        .command(`applet-info`, `Get applet info`, {
        username: {
            type: `string`,
            alias: `u`,
            demandOption: true,
            description: `User name`,
        },
        password: {
            type: `string`,
            alias: `p`,
            demandOption: true,
            description: `Password`,
        },
        appletId: {
            type: `string`,
            alias: `a`,
            description: `Applet Id`,
            demandOption: true,
        },
    }, async (argv) => {
        const { username, password, outFile, appletId } = zod_1.z
            .object({
            username: zod_1.z.string(),
            password: zod_1.z.string(),
            outFile: zod_1.z.string().optional(),
            appletId: zod_1.z.string(),
        })
            .parse(argv);
        const client = await _1.Client.createClient(username, password);
        await print(outFile, await client.getAppletInfo(appletId));
    })
        .command(`applet-data`, `Get applet data`, {
        username: {
            type: `string`,
            alias: `u`,
            demandOption: true,
            description: `User name`,
        },
        password: {
            type: `string`,
            alias: `p`,
            demandOption: true,
            description: `Password`,
        },
        appletId: {
            type: `string`,
            alias: `a`,
            description: `Applet Id`,
            demandOption: true,
        },
    }, async (argv) => {
        const { username, password, outFile, appletId } = zod_1.z
            .object({
            username: zod_1.z.string(),
            password: zod_1.z.string(),
            outFile: zod_1.z.string().optional(),
            appletId: zod_1.z.string(),
        })
            .parse(argv);
        const client = await _1.Client.createClient(username, password);
        await print(outFile, await client.getAppletData(appletId));
    })
        .command(`decrypt-applet-data`, `Decrypt applet data`, {
        appletPassword: {
            type: `string`,
            alias: `p`,
            description: `Applet password`,
            demandOption: true,
        },
        appletInfo: {
            type: `string`,
            alias: `i`,
            description: `Applet info file`,
            demandOption: true,
        },
        appletData: {
            type: `string`,
            alias: `d`,
            description: `Applet data file`,
            demandOption: true,
        },
    }, async (argv) => {
        const { outFile, appletPassword, appletInfo: appletInfoFile, appletData: appletDataFile, } = zod_1.z
            .object({
            outFile: zod_1.z.string().optional(),
            appletPassword: zod_1.z.string(),
            appletInfo: zod_1.z.string(),
            appletData: zod_1.z.string(),
        })
            .parse(argv);
        const appletInfo = await promises_1.readFile(path_1.resolve(process.cwd(), appletInfoFile), {
            encoding: `utf-8`,
        }).then((appletInfo) => Responses_1.AppletInfo.parse(JSON.parse(appletInfo)));
        const appletData = await promises_1.readFile(path_1.resolve(process.cwd(), appletDataFile), {
            encoding: `utf-8`,
        }).then((appletData) => Responses_1.AppletData.parse(JSON.parse(appletData)));
        const appletResponses = await Encryption_1.decryptAppletResponses(appletInfo, appletData, appletPassword);
        await print(outFile, JSON.stringify(appletResponses, null, 2));
    })
        .help()
        .strict()
        .demandCommand(1)
        .parseAsync();
};
main().catch((error) => {
    console.error(error);
    process.exit(1);
});
//# sourceMappingURL=cli.js.map